package gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.impl;

import java.util.Map;

import javax.annotation.Resource;

import org.apache.log4j.Logger;

import firstdatabank.database.FDBDataManager;
import firstdatabank.database.FDBException;
import firstdatabank.dif.DAMScreenResult;
import firstdatabank.dif.DAMScreenResults;
import firstdatabank.dif.ScreenAllergies;
import firstdatabank.dif.ScreenAllergy;
import firstdatabank.dif.ScreenDrugs;
import firstdatabank.dif.Screening;
import gov.va.med.pharmacy.peps.common.exception.InterfaceException;
import gov.va.med.pharmacy.peps.common.vo.AllergyCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.DrugCheckResultsVo;
import gov.va.med.pharmacy.peps.common.vo.DrugCheckVo;
import gov.va.med.pharmacy.peps.common.vo.ScreenAllergyVo;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.PerformAllergyCheckCapability;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.utility.MessageConversionUtility;

/**
 * The class responsible for actually performing the Allergy checks utilizing the FDB DIF API.
 * 
 * @author vhaisaduffg
 *
 */
public final class PerformAllergyCheckCapabilityImpl implements PerformAllergyCheckCapability {

    private static final Logger LOG = Logger
            .getLogger(PerformDrugDrugCheckCapabilityImpl.class);

    @Resource
    private Screening screening;
    @Resource
    private FDBDataManager fdbDataManager;

    @Override
    public DrugCheckResultsVo<AllergyCheckResultVo> performAllergyCheck(ScreenDrugs screenDrugs, 
            Map<String, DrugCheckVo> drugMap,
            ScreenAllergies allergens, boolean prospectiveOnly) {
        try {
        
            DAMScreenResults results = screening.DAMScreen(screenDrugs, allergens, prospectiveOnly);
            return convertResults(results, screenDrugs, allergens, drugMap);
        }
        catch (FDBException e) {
            LOG.error(e.getMessage(), e);
            throw new InterfaceException(e, InterfaceException.INTERFACE_ERROR, InterfaceException.DRUG_DATA_VENDOR);
        }
    }
    
    /**
     * Converts the DAMScreenResults object from FDB in to a DrugCheckResultsVo object for MOCHA Server.
     * 
     * @param results the DAMScreenResults
     * @param screenDrugs the ScreenDrugs object
     * @param screenAllergies the ScreenAllergies object
     * @param drugMap the DrugCheckVo map
     * @return DrugCheckResultsVo
     */
    private DrugCheckResultsVo<AllergyCheckResultVo> convertResults(DAMScreenResults results, ScreenDrugs screenDrugs, 
            ScreenAllergies screenAllergies, Map<String, DrugCheckVo> drugMap) {
        DrugCheckResultsVo<AllergyCheckResultVo> convertedResults = new DrugCheckResultsVo<AllergyCheckResultVo>();

        convertedResults.setMessages(MessageConversionUtility.toDrugCheckMessages(results.getMessages(),
                drugMap, screenDrugs));
        
        final int resultsLen = results.count();
        for ( int i = 0; i < resultsLen; i++ ) {
            AllergyCheckResultVo checkResult = new AllergyCheckResultVo();
            DAMScreenResult screenResult = results.item(i);
            checkResult.setAllergenCustomId(screenResult.getAllergenCustomID());
            checkResult.setAllergenDescription(screenResult.getAllergenDescription());
            checkResult.setAllergenInternalId(screenResult.getAllergenInternalID());
            checkResult.setDrugCustomId(screenResult.getDrugCustomID());
            checkResult.setDrugDescription(screenResult.getDrugDescription());
            checkResult.setDrugInternalId(screenResult.getDrugInternalID());
            checkResult.setHitBasedOnInactiveHicl(screenResult.getHitBasedOnInactiveHICL());
            checkResult.setHitBasedOnInactiveIngredientInformation(screenResult.getHitBasedOnInactiveIngredientInformation());
            checkResult.setInactiveHiclSeqNo(screenResult.getInactiveHICLSeqNo());
            //checkResult.setInternationalMatchDescription();
            checkResult.setMatchDescription(screenResult.getMatchDescription());
            checkResult.setMatchId(screenResult.getMatchID());
            checkResult.setMatchType(screenResult.getMatchType().toString());
            checkResult.setReaction(screenResult.getReaction());
            checkResult.setScreenDrug(drugMap.get(screenDrugs.get(screenResult.getDrugIndex()).getCustomID()));
            checkResult.setScreeningAllergy(toScreenAllergy(screenAllergies.item(screenResult.getAllergenIndex())));
            checkResult.setScreenMessage(screenResult.getScreenMessage());
            convertedResults.getChecks().add(checkResult);
        }
        
        return convertedResults;
    }
    
    private ScreenAllergyVo toScreenAllergy(ScreenAllergy screenAllergy) {
        ScreenAllergyVo vo = new ScreenAllergyVo();
        
        //vo.setClinicalAllergenId(screenAllergy.getClinicalAllergenID());
        vo.setDescription(screenAllergy.getDescription());
        vo.setId(Long.valueOf(screenAllergy.getID()));
        vo.setIdType(screenAllergy.getIDType().toString());
        vo.setInternalId(screenAllergy.getInternalID());
        vo.setInternationalDescription(screenAllergy.getIntlDescription().toString());
        vo.setProspective(screenAllergy.getProspective());
        vo.setReaction(screenAllergy.getReaction());
        vo.setTallManDescription(screenAllergy.getTallManDescription());
        vo.setTallManInd(screenAllergy.getTallManInd());
        
        return vo;
    }

    /**
     * Returns the Screening object.
     * 
     * @return the screening
     */
    public Screening getScreening() {
        return screening;
    }

    /**
     * Sets the Screening object.
     * 
     * @param screening the screening to set
     */
    public void setScreening(Screening screening) {
        this.screening = screening;
    }

    /**
     * Returns the FDBDataManager.
     * 
     * @return the fdbDataManager
     */
    public FDBDataManager getFdbDataManager() {
        return fdbDataManager;
    }

    /**
     * Sets the FDBDataManager.
     * 
     * @param fdbDataManager the fdbDataManager to set
     */
    public void setFdbDataManager(FDBDataManager fdbDataManager) {
        this.fdbDataManager = fdbDataManager;
    }

}
